import { Component, OnInit } from "@angular/core";
import { FormBuilder, FormControl, FormGroup, Validators } from "@angular/forms";
import { ActivatedRoute, Router } from "@angular/router";
import { catchError, finalize } from "rxjs/operators";
import { AuthErrorMessages } from "src/app/models/auth/auth-error-messages.enum";
import { AuthUserDTO } from "src/app/models/auth/auth-user.dto";
import { AuthUserAtrErrors } from "src/app/models/auth/auth-user-atr-errors";
import { ErrorDTO } from "src/app/models/error/error-dto";
import { ServiceErrorCode } from "src/app/models/error/service-error-code.enum";
import { AuthService } from "src/app/services/auth/auth.service";
import { SpinnerService } from "src/app/services/spinner/spinner.service";
import { throwError } from "rxjs";
import { OAuthDTO } from "../../../../models/auth/oauth.dto";

@Component({
  selector: "app-registration",
  templateUrl: "./registration.component.html",
  styleUrls: ["./registration.component.scss"]
})
export class RegistrationComponent implements OnInit {

  loginForm!: FormGroup<{
    firstName: FormControl<string | null>,
    lastName: FormControl<string | null>,
    email: FormControl<string | null>,
    emailRepeat: FormControl<string | null>,
    password: FormControl<string | null>,
    passwordRepeat: FormControl<string | null>
  }>;
  submitted = false;
  authUserAtrErrors!: AuthUserAtrErrors;
  repeatErrors = {
    email: false,
    password: false
  };

  showEMailConfirmation = false;
  showEMailConfirmationError = false;

  code!: string;
  state!: string;
  user!: AuthUserDTO;
  oAuthId!: string;

  constructor(
    private authService: AuthService,
    private formBuilder: FormBuilder,
    private router: Router,
    private spinnerService: SpinnerService,
    private route: ActivatedRoute
  ) {
    this.spinnerService.showSpinner();
    this.authService.isUserLoggedInAsync().then(res => {
      if (res) {
        this.router.navigate(["/dashboard"]);
      }
      this.spinnerService.hideSpinner();
    });
  }

  ngOnInit(): void {
    this.initData();
    this.spinnerService.showSpinner();
    this.route.queryParams.pipe(catchError(err => {
      this.spinnerService.hideSpinner();
      this.router.navigate(["auth", "login"]).then(() => {
      });
      return throwError(() => err);
    })).subscribe(params => {
        if (!params["code"] || !params["state"]) {
          this.spinnerService.hideSpinner();
          return;
        }

        if (this.user) {
          this.spinnerService.hideSpinner();
          return;
        }

        this.code = params["code"];
        this.state = params["state"];
        this.authService.discordCreateUser(this.code, this.state).pipe(catchError(err => {
          this.spinnerService.hideSpinner();
          this.router.navigate(["auth", "login"]).then(() => {
          });
          return throwError(() => err);
        })).subscribe(oAuthDTO => {
            if (oAuthDTO) {
              this.user = oAuthDTO.user;
              this.oAuthId = oAuthDTO.oAuthId;
            }
            if (this.user && !this.oAuthId) {
              this.router.navigate(["auth", "login"]).then(() => {
              });
              return;
            }

            this.router.navigate(["auth", "register"]).then(() => {
            });
            this.initData();
          }
        );
        this.initData();
      }
    );
  }

  initData() {
    this.initLoginForm();
    this.resetStateFlags();
    this.confirmEMail();
    this.spinnerService.hideSpinner();
  }

  resetStateFlags(): void {
    this.authUserAtrErrors = new AuthUserAtrErrors();
    this.repeatErrors = {
      email: false,
      password: false
    };
  }

  login(): void {
    this.router.navigate(["/auth/login"]);
  }

  initLoginForm(): void {
    this.loginForm = this.formBuilder.group({
      firstName: [this.user ? this.user.firstName : "", Validators.required],
      lastName: [this.user ? this.user.lastName : "", Validators.required],
      email: [this.user ? this.user.email : "", [Validators.required, Validators.email]],
      emailRepeat: [this.user ? this.user.email : "", [Validators.required, Validators.email]],
      password: [this.user ? this.user.password : "", [Validators.required, Validators.minLength(8)]],
      passwordRepeat: [this.user ? this.user.password : "", [Validators.required, Validators.minLength(8)]]
    });

    if (this.user) {
      this.loginForm.controls.email.disable();
      this.loginForm.controls.emailRepeat.disable();
    }
  }

  register(): void {
    this.submitted = true;
    this.resetStateFlags();

    // stop here if form is invalid
    if (this.loginForm.invalid) {
      return;
    }

    if (this.loginForm.value.email !== this.loginForm.value.emailRepeat) {
      this.repeatErrors.email = true;
      return;
    }

    if (this.loginForm.value.password !== this.loginForm.value.passwordRepeat) {
      this.repeatErrors.password = true;
      return;
    }

    this.spinnerService.showSpinner();
    if (this.user && this.oAuthId) {
      this.registerWithOAuth();
      return;
    }

    this.registerLocal();
  }

  private registerWithOAuth() {
    const oAuthDTO: OAuthDTO = {
      user: {
        firstName: this.loginForm.value.firstName ?? this.user.firstName,
        lastName: this.loginForm.value.lastName ?? this.user.lastName,
        email: this.loginForm.value.email ?? this.user.email,
        password: this.loginForm.value.password ?? null
      },
      oAuthId: this.oAuthId
    };
    this.authService.discordRegister(oAuthDTO)
      .pipe(catchError(error => {
        if (error.error !== null) {
          const err: ErrorDTO = error.error;

          if (err.errorCode === ServiceErrorCode.InvalidUser && err.message === AuthErrorMessages.UserAlreadyExists) {
            this.authUserAtrErrors.email.wrongData = true;
          }
        }
        this.spinnerService.hideSpinner();
        throw error;
      }))
      .subscribe(resp => {
        this.spinnerService.hideSpinner();
        this.router.navigate(["/auth/login"]);
      });
  }

  private registerLocal() {
    const user: AuthUserDTO = {
      firstName: this.loginForm.value.firstName ?? null,
      lastName: this.loginForm.value.lastName ?? null,
      email: this.loginForm.value.email ?? null,
      password: this.loginForm.value.password ?? null
    };
    this.authService.register(user)
      .pipe(catchError(error => {
        if (error.error !== null) {
          const err: ErrorDTO = error.error;

          if (err.errorCode === ServiceErrorCode.InvalidUser && err.message === AuthErrorMessages.UserAlreadyExists) {
            this.authUserAtrErrors.email.wrongData = true;
          }
        }
        this.spinnerService.hideSpinner();
        throw error;
      }))
      .subscribe(resp => {
        this.spinnerService.hideSpinner();
        this.router.navigate(["/auth/login"]);
      });
  }

  confirmEMail(): void {
    const id = this.route.snapshot.params["id"];
    if (id) {
      this.spinnerService.showSpinner();
      this.authService.confirmEMail(id)
        .pipe(catchError(error => {
          this.router.navigate(["/auth/login"]);
          this.spinnerService.hideSpinner();
          throw error;
        }))
        .subscribe(resp => {
          this.spinnerService.hideSpinner();
          this.router.navigate(["/auth/login"]);
        });
    }
  }

  discordLogin() {
    this.authService.getDiscordAuthURL().subscribe(url => {
      window.location.href = url.loginUrl;
    });
  }
}
